defmodule Baz.Repo.Migrations.CreateCollectionEvents do
  use Ecto.Migration

  def up do
    create_query = """
    CREATE TYPE event_type AS ENUM (
      'item_listed',
      'item_sold',
      'item_transferred',
      'item_metadata_update',
      'item_canceled',
      'item_received_offer',
      'item_received_bid',
      'created',
      'successful',
      'collection_offer',
      'trait_offer',
      'bid_entered',
      'cancelled',
      'transfer'
    )
    """
    execute(create_query)

    create table(:collection_events, primary_key: false) do
      add :event_timestamp, :utc_datetime_usec, null: false
      add :event_type, :event_type, null: false
      add :venue, :text, null: false
      add :slug, :text, null: false
      add :token_id, :integer

      timestamps()
    end

    execute "SELECT create_hypertable('#{collection_events_table_name(table_and_index_prefix)}', 'event_timestamp');"
  end

  def down do
    drop table(:collection_events)
    execute("DROP TYPE event_type")
  end

  @collection_events_base_table_name "collection_events"
  defp collection_events_table_name(table_and_index_prefix) do
    if table_and_index_prefix == nil do
      @collection_events_base_table_name
    else
      "#{table_and_index_prefix}_#{@collection_events_base_table_name}"
    end
  end
end
